/*
 * util.c	Utility functions for spwauth
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <syslog.h>
#include <fcntl.h>
#include <paths.h>
#include "sha.h"

#define SPWA_PRIVATE
#include "spwauth.h"

#define CRYPT_BYTES	32
#ifndef _PATH_RANDOM
# define _PATH_RANDOM	"/dev/random"
#endif

/*
 * Encrypt password according to JFH gospel
 */
char *
spcrypt(char *pass, char *salt)
{
	static char	crypted[CRYPT_BYTES];

	strcpy(crypted, crypt(pass, salt));
	if (strlen(pass) > 8)
		strcat(crypted, crypt(pass + 8, salt) + 2);
	return crypted;
}

/*
 * Lookup shadow entry. For now, we just look into /etc/shadow.
 * A full implementation will want to place a call to ypserv on
 * 127.0.0.1 here.
 */
#if 1
struct spwd *
splookup(char *domain, char *user)
{
	return getspnam(user);
}
#else
struct spwd *
splookup(char *domain, char *user)
{
	static struct spwd	spw;
	struct passwd		*pw;

	if (!(pw = getpwnam(user)))
		return NULL;
	spw.sp_namp = pw->pw_name;
	spw.sp_pwdp = pw->pw_passwd;
	return &spw;
}
#endif

/*
 * compute SHA of seed concatenated with encrypted password
 */
void
compute_hash(unsigned char *result, unsigned char *seed, char *cr, int len)
{
	unsigned char	buffer[2 * HASH_BYTES + CRYPT_BYTES];
	SHA_INFO	ctxt;

	memset(buffer, 0, sizeof(buffer));
	memcpy(buffer, seed, HASH_BYTES);
	memcpy(buffer + HASH_BYTES, cr, len);

	sha_init(&ctxt);
	sha_update(&ctxt, buffer, HASH_BYTES + len);
	sha_final(&ctxt);
	memcpy(result, ctxt.digest, HASH_BYTES);
#ifdef DEBUG
	printf("compute_hash\n"
		"seed:\t%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x\n"
		"crypt:\t%s (%d)\n"
		"reslt:\t%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x\n",
		seed[0], seed[1], seed[2], seed[3], seed[4],
		seed[5], seed[6], seed[7], seed[8], seed[9],
		seed[10], seed[11], seed[12], seed[13], seed[14],
		seed[15], seed[16], seed[17], seed[18], seed[19],
		cr, len,
		result[0], result[1], result[2], result[3], result[4],
		result[5], result[6], result[7], result[8], result[9],
		result[10], result[11], result[12], result[13], result[14],
		result[15], result[16], result[17], result[18], result[19]);
#endif
}

/*
 * Random number generation
 *
 * This implementation uses Linux's /dev/random, which is OK for demo
 * purposes, but not very portable. A good RNG should do this:
 *   1	Seed the RNG with lots of noise at boot time, e.g. by computing
 *	SHA(/dev/kmem). This value will be called R0.
 *   2	When extracting a random number, return SHA(R0, usecs)
 *   3	Every M seconds or N requests (whichever comes first), re-initialize
 *	the RNG by performing step 1.
 */
void
get_random(unsigned char *number)
{
	int	fd, len, have = 0;

	if ((fd = open(_PATH_RANDOM, O_RDONLY)) < 0) {
		syslog(LOG_ERR, "Can't open %s: %m", _PATH_RANDOM);
		exit(2);
	}
	do {
		len = read(fd, number + have, HASH_BYTES - have);
		if (len < 0) {
			syslog(LOG_ERR, "Read error on %s: %m", _PATH_RANDOM);
			exit(2);
		}
		have += len;
	} while (have != HASH_BYTES);
	close(fd);
}
